(* Delphi XE2 program
   test fpr file copy routines

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   last changed: May 2015
   *)

unit CopyFMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Prgrss, ComCtrls, ExtFileTools, FtStrings,
  Indicators, NumberEd;

type

  TForm1 = class(TForm)
    Label1: TLabel;
    VerzBtn: TSpeedButton;
    VerzEdit: TEdit;
    Label2: TLabel;
    FileBtn: TSpeedButton;
    FName: TEdit;
    btnThCopy: TButton;
    OpenDialog: TOpenDialog;
    btnThGz: TButton;
    btnPause: TSpeedButton;
    BarProgress: TBarProgress;
    trbPrior: TTrackBar;
    Label3: TLabel;
    btnThZip: TButton;
    SpotIndicator: TSpotIndicator;
    btnUnzip: TButton;
    Label6: TLabel;
    CopyTime: TFloatEdit;
    Label7: TLabel;
    GroupBox1: TGroupBox;
    btnQuit: TButton;
    Label4: TLabel;
    btnShellCopy: TButton;
    cbPerm: TCheckBox;
    btnCopy: TButton;
    reBSize: TRangeEdit;
    udBSize: TNumUpDown;
    procedure FileBtnClick(Sender: TObject);
    procedure VerzBtnClick(Sender: TObject);
    procedure btnThCopyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnThGzClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnThZipClick(Sender: TObject);
    procedure btnUnzipClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure btnShellCopyClick(Sender: TObject);
  private
    { Private-Deklarationen }
    IniName : string;
    CopyThread       : TCopyThread;
    STime            : TDateTime;
    function WaitForThread : integer;
    procedure ShowStatus (Start : boolean);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses StringUtils, WinUtils, UnicodeUtils, FileUtils, ShellDirDlg, DateUtils,
  WinShell, WinApiUtils, WinApi.AccCtrl, WinApi.AclApi;

procedure TForm1.FormCreate(Sender: TObject);
var
  IniFile : TUnicodeIniFile;
begin
  IniName:=NewExt(Application.ExeName,'ini');
  IniFile:=TUnicodeIniFile.CreateForRead (iniName);
  with IniFile do begin
    FName.Text:=ReadString('Config','Filename','');
    VerzEdit.Text:=ReadString('Config','Directory','');
    Free;
    end;
  end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  IniFile : TUnicodeIniFile;
begin
  IniFile:=TUnicodeIniFile.CreateForWrite (iniName);
  with IniFile do begin
    WriteString('Config','Filename',FName.Text);
    WriteString('Config','Directory',VerzEdit.Text);
    Free;
    end;
  end;

procedure TForm1.FormShow(Sender: TObject);
begin
  ShellDirDialog.LoadFromIni(IniName,'History');
  end;

procedure TForm1.FileBtnClick(Sender: TObject);
begin
  with Opendialog do begin
    if Execute then begin
      if Files.Count>0 then FName.Text:=Files.CommaText
      else FName.Text:=FileName;
      end;
    end;
  end;

procedure TForm1.VerzBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=VerzEdit.Text;
  if ShellDirDialog.Execute ('Ziel-Verzeichnis auswhlen',true,true,'C:\',s) then begin
    VerzEdit.Text:=s;
    end;
  end;

function TForm1.WaitForThread : integer;
begin
  with (CopyThread as TWinCopyThread) do begin
    CopyPermissions:=cbPerm.Checked;
    Resume;
    repeat
      Sleep(0);
      Application.ProcessMessages;
      if btnPause.Down then begin
        if not Suspended then Suspend;
        end
      else begin
        if Suspended then Resume;
        end;
      until not btnPause.Down and Done;
    Result:=ErrorCode;
    Free;
    end;
  end;

procedure TForm1.ShowStatus (Start : boolean);
begin
  if Start then begin
    CopyTime.Value:=0;
    BarProgress.Status:=psOn;
    SpotIndicator.Indicate:=true;
    btnPause.Show;
    Application.ProcessMessages;
    STime:=Now;
    end
  else begin
    CopyTime.Value:=MilliSecondsBetween(STime,Now)/1000;
    btnPause.Hide;
    SpotIndicator.Indicate:=false;
    with BarProgress do begin
      Status:=psOff; Reset;
      end;
    end;
  end;

procedure TForm1.btnThCopyClick(Sender: TObject);
var
  s,t : string;
  ec  : integer;
begin
  ShowStatus(true);
  s:=FName.Text;
  repeat
    t:=ReadNxtQuotedStr(s,',','"');
    CopyThread:=TWinCopyThread.Create(t,SetDirName(VerzEdit.Text)+ExtractFileName(t),
         nil,[],true,reBSize.Value*1024,TThreadPriority(trbPrior.Position));
    ec:=WaitForThread;
    if ec<>errOK then ErrorDialog('',GetCopyErrMsg(ec));
    until length(s)=0;
  ShowStatus(false);
  end;

procedure TForm1.btnCopyClick(Sender: TObject);
var
  s,t,u : string;
  ok    : boolean;
  DACL           : PACL;
  sd             : PSecurityDescriptor;
begin
  ShowStatus(true);
  s:=FName.Text;
  repeat
    t:=ReadNxtQuotedStr(s,',','"');
    u:=SetDirName(VerzEdit.Text)+ExtractFileName(t);
    try
      CopyFileTS(t,u,-1,reBSize.Value*1024);
    except
      on E:ECopyError do ErrorDialog('',E.Message+#13'Error: '+IntToStr(E.CopyError));
      end;
    if cbPerm.Checked then begin
      ok:=CopyFileAcl(t,u)=ERROR_SUCCESS;
      if not ok then ErrorDialog('Fehler beim Kopieren der Berechtigungen');
      end;
    until length(s)=0;
  ShowStatus(false);
  end;

procedure TForm1.btnThGzClick(Sender: TObject);
var
  s,t : string;
  ec  : integer;
begin
  ShowStatus(true);
  s:=FName.Text;
  repeat
    t:=ReadNxtQuotedStr(s,',','"');
    CopyThread:=TWinCopyThread.Create(t,SetDirName(VerzEdit.Text)+ExtractFileName(t)+'.gz',
       nil,[coCompress],false,reBSize.Value*1024,TThreadPriority(trbPrior.Position));
    ec:=WaitForThread;
    if ec<>errOK then ErrorDialog('',GetCopyErrMsg(ec));
    until length(s)=0;
  ShowStatus(false);
  end;

procedure TForm1.btnUnzipClick(Sender: TObject);
var
  GzInfo       : TGzFileInfo;
  sn           : string;
begin
  ShowStatus(true);
  if GzFileInfo(FName.Text,GzInfo) then begin
    sn:=GzInfo.Filename;
    CopyThread:=TGUnZipThread.Create(FName.Text,SetDirName(VerzEdit.Text)+sn,
       nil,false,reBSize.Value*1024,TThreadPriority(trbPrior.Position));
    with CopyThread do begin
      repeat Application.ProcessMessages until Done;
      Free;
      end;
    end;
  ShowStatus(false);
  end;

procedure TForm1.btnThZipClick(Sender: TObject);
var
  s,t : string;
begin
  ShowStatus(true);
  s:=FName.Text;
  with TZipWrite.Create(SetDirName(VerzEdit.Text)+'Test.zip',ExtractFilePath(ReadNxtStr(s,',')),
       '',0,reBSize.Value*1024) do begin
    s:=FName.Text;
    repeat
      t:=ReadNxtQuotedStr(s,',','"');
      AddFile(t);
      until length(s)=0;
    WriteDirectory;
    Free;
    end;
  ShowStatus(false);
  end;

{ ------------------------------------------------------------------- }
procedure TForm1.btnPauseClick(Sender: TObject);
begin
  with CopyThread do if btnPause.Down then Suspend else Resume;
  BarProgress.Pause(btnPause.Down);
//  Priority:=tpLower else Priority:=tpHigher;
//  CopyThread.Suspended:=SpeedButton1.Down;
  end;

procedure TForm1.btnQuitClick(Sender: TObject);
begin
  Close;
  end;

procedure TForm1.btnShellCopyClick(Sender: TObject);
begin
  ShowStatus(true);
  if ShellCopyFiles(Application.Handle,FName.Text,SetDirName(VerzEdit.Text)+ExtractFileName(FName.Text),'Kopiere',false)<>NO_ERROR
    then ErrorDialog('','Shell Copy Error');
  ShowStatus(false);
  end;

end.
